PROGRAM Compute_Ql
  USE cosmo
  ! A program for computing the cross-power spectrum of CMB lensing potential
  ! and the Rees-Sciama effect, Q(l). It reads in the non-linear power spectrum
  ! table generated by "generate_pkz" or "generate_pkz3pt" (also available on CRL). 
  ! The output file (ell_ql_***.txt) contains:
  ! (1st column) multipole
  ! (2nd column) Q(l)
  ! January 2, 2012: E.Komatsu
  IMPLICIT none
  character(len=128) :: filename
  double precision :: lmax=3000d0 ! maximum multipole
  double precision :: lmin=5d1 ! minimum multipole
  integer :: nz=901 ! # of data points in redshift in the input data file
  integer :: nk=691 ! # of data points in wavenumber in the input data file
  double precision :: zmax=9d0 ! maximum redshift in the input data file
  double precision, allocatable, dimension(:,:) :: pkz,y2a
  double precision, allocatable, dimension(:) :: redshift,k_ov_h
  double precision :: dlnPkdz,lnPk,kout_ov_h,zout,dln1pz
  double precision :: Ppsi,rombint,rz,rstar,d_chi,ell,dlnell,Ql,akmax,akmin
  integer :: ik,iz,j
  external rombint,d_chi
! Specify four cosmological parameters
! The data type has been defined in MODULE cosmo.
  ode0=0.723d0
  om0=0.277d0
  h0=0.702d0
  w=-1d0
! read in and tabulate P(k,z)
  ALLOCATE(pkz(nz,nk),y2a(nz,nk),redshift(nz),k_ov_h(nk))
  print*,'Which non-linear model do you use?'
  print*,'1: 3rd-order perturbation theory (3PT)'
  print*,'2: Halofit [Smith et al., (2003)]'
  print*,'3: Ma et al. (1999)'
  print*,'4: Peacock&Dodds (1996)'
  read*,j
  select case (j)
     case (1)
        filename='redshift_wavenumber_pk3pt.txt'
     case (2)
        filename='redshift_wavenumber_pkhalofit.txt'
     case (3)
        filename='redshift_wavenumber_pkma99.txt'
     case (4)
        filename='redshift_wavenumber_pkpd96.txt'
  end select
  print*,'read in '//trim(filename)//' and generate an interpolation table'
  open(1,file=filename,status='old')
  do iz=1,nz
     do ik=1,nk
        read(1,*)redshift(iz),k_ov_h(ik),pkz(iz,ik)
     enddo
  enddo
  close(1)
  akmin=MINVAL(k_ov_h)
  akmax=MAXVAL(k_ov_h)
  print*,'minimum wavenumber read in=',akmin,' h/Mpc'
  print*,'maximum wavenumber read in=',akmax,' h/Mpc'
  CALL splie2(redshift,dlog(k_ov_h),dlog(pkz),nz,nk,y2a)
  print*,'done. now computing...'
  select case (j)
     case (1)
        filename='ell_ql_3pt.txt'
     case (2)
        filename='ell_ql_halofit.txt'
     case (3)
        filename='ell_ql_ma99.txt'
     case (4)
        filename='ell_ql_pd96.txt'
  end select
  open(2,file=filename,status='unknown')
  rstar=rombint(d_chi,1d0/1092d0,1d0,1d-6) !comoving distance to z*=1091 [Mpc/h]
  !!$ print*,'r*=',rstar/h0,'Mpc'
  dlnell=dlog(lmax/lmin)/2d2  ! dlog(lmax/lmin)/(# of points in ell)
  do j=1,201
     ell=lmin*dexp(dble(j-1)*dlnell)
     print*,'l=',ell
     Ql=0d0
     ! integration over redshift
     do iz=1,100
        if((iz.eq.1).or.(iz.eq.100))then
           dln1pz=0.5d0*dlog((1d0+zmax))/1d2 ! for the integration boundary
        else
           dln1pz=dlog((1d0+zmax))/1d2 ! dlog(1+zmax)/(# of points in redshift)
        endif
        zout=dexp(dble(iz)*dln1pz)-1d0
        rz=rombint(d_chi,1d0/(1d0+zout),1d0,1d-6) ! h^-1 Mpc
        kout_ov_h=ell/rz ! h/Mpc
        if(kout_ov_h<akmin)then
           print*,kout_ov_h,'is smaller than ',akmin,' Mpc/h.'
           stop
        else if(kout_ov_h>akmax)then
           print*,kout_ov_h,'is greater than ',akmax,' Mpc/h.'
           stop
        endif
        CALL splin2(redshift,dlog(k_ov_h),dlog(pkz),y2a,nz,nk, &
             zout,dlog(kout_ov_h),lnPk)
        Ppsi=(1.5d0*om0)**2d0*(1d0/2998d0/kout_ov_h)**4d0 &
             *dexp(lnPk)*(1d0+zout)**2d0!P(k) of Newtonian potential [h^-3 Mpc^3]
        CALL splin2_der1(redshift,dlog(k_ov_h),dlog(pkz),y2a,nz,nk, &
             zout,dlog(kout_ov_h),dlnPkdz) ! derivative with respect to redshift
        Ql=Ql+2d0*(rstar-rz)/rstar/rz**3d0*Ppsi*(dlnPkdz+2d0/(1d0+zout)) &
             *(1d0+zout)*dln1pz ! cross-power spectrum of lensing-RS effect
     enddo
     print*,'Ql=',Ql
     write(2,'(2E18.8)')ell,Ql
  enddo
  close(2)
  DEALLOCATE(pkz,y2a,redshift,k_ov_h)
END PROGRAM Compute_Ql
!-----------------------------------------------------------------------------
DOUBLE PRECISION FUNCTION d_chi(x)
! d_chi(x) = 1/[H(x)*x^2] in units of h^-1 Mpc
  USE cosmo
  IMPLICIT none
  DOUBLE PRECISION, intent(IN) :: x
  DOUBLE PRECISION :: tcmb=2.725d0
  DOUBLE PRECISION :: ok0,ogh2,orh2,or0,neff=3.04d0
  ogh2 = 2.469d-5*(tcmb/2.725)**4.
  orh2 = (1.+0.2271073*neff)*ogh2
  or0  = orh2/h0**2.
  ok0  = 1.-om0-ode0 ! don't include or0 in curvature.
  d_chi = 2998d0/sqrt(om0*x+or0+ok0*x**2.+ode0*x**(1.-3.*w))
  return
END FUNCTION d_chi
